const js2xmlparser = require('js2xmlparser')
const convert = require('xml-js')

/**
 * A module containing common javascript util methods
 *
 * `const util = require('nov-nodejs-core-util');`
 * @module util
 */

/**
 * Converts any `object` to base64 string representation
 * @example
 * // returns dGVzdA==
 * const util = require('nov-nodejs-core-util');
 * util.to_base64('test');
 *
 * @returns {string} Base64 string representation of the input
 * @param {object|string} input - Input reqired to be converted to base64
 */
exports.to_base64 = (input) => {
  if (typeof input === 'object') {
    input = JSON.stringify(input)
  }

  // eslint-disable-next-line new-cap
  return input ? new Buffer.from(input.toString()).toString('base64') : ''
}

/**
 * Converts a js object or valid JSON string to xml
 * @example
 * // returns <?xml version='1.0'?>
 * //         <root>
 * //           <test>1</test>
 * //         </root>
 * const util = require('nov-nodejs-core-util');
 * util.json_to_xml({ test: 1 }, 'root');
 *
 * @returns {string} xml representation of the input JSON string or js object
 * @param {object|string} json - Javascript object or JSON string
 * @param {string} rootElement - Optional string to provide as the root element
 */
exports.json_to_xml = (json, rootElement) => {
  if (typeof json !== 'object') {
    json = JSON.parse(json)
  }
  return js2xmlparser.parse(rootElement || 'root', json)
}

/**
 * Converts a XML string to a JSON string
 * @example
 * // returns "{\"x\":{\"test\":{\"_text\":\"1\"}}}"
 * const util = require('nov-nodejs-core-util');
 * util.xml_to_json("<x><test>1</test></x>");
 *
 * @returns {string} JSON string representation of the xml input
 * @param {string} xml - XML string
 */
exports.xml_to_json = (xml) => {
  return convert.xml2json(xml, { compact: true })
}
